---------------
-- Namespace --
---------------
local LDB = LibStub:GetLibrary("LibDataBroker-1.1")
local tip = LibStub:GetLibrary("LibTooltip-1.0"):Acquire("nanoTalk", 3, "LEFT", "CENTER", "CENTER")


nanoTalk = CreateFrame("Frame")
nanoTalk.obj = LDB:NewDataObject("nanoTalk",
				 {
					 type	= "data source",
					 icon	= "Interface\\GossipFrame\\GossipGossipIcon",
					 label	= "nanoTalk",
					 text	= "0t",
				 }
			 )


nanoTalkVersion = GetAddOnMetadata("nanoTalk", "Version")


local nanoTalkDefaults = {
	ChatNotify = true,
	NotifyAFK = true,
	TalkTime = 120,
	TrackNum = 10,
	TrackTime = 1800,
	Version = nanoTalkVersion
}
nanoTalkConfig = nanoTalkConfig or nanoTalkDefaults


local updatedLines = {}
local timestamps = {}
local people = {}
local AFKtime = 0

---------------
-- Functions --
---------------
local function GetCount(timer)
	if (timer == nil) then
		return #timestamps
	end

	local count = 0

	for _, tm in ipairs(timestamps) do
		if ((GetTime() - tm) <= timer) then
			count = count + 1
		end
	end
	return count
end


local function TimeStr(seconds)
	local tm = seconds
	local timeval = tostring(math.floor(tm % 60)).."s"

	if (tm >= 60) then 
		tm = tm / 60
		timeval = tostring(math.floor(tm % 60)).."m"..timeval

		if (tm >= 60) then
			tm = tm / 60
			timeval = tostring(math.floor(tm % 24)).."h"..timeval

			if (tm >= 24) then
				tm = tm / 24
				timeval = tostring(math.floor(tm)).."d"..timeval
			end
		end
	end
	return timeval
end


local function ShowTooltip()
	tip:Clear()
	tip:AddHeader("nanoTalk")
	tip:AddLine(" ")

	local count = GetCount(nanoTalk.trackTime)
	local linenum

	if (count > 0) then
		tip:AddHeader("|cffff9933Name|r", "|cffff9933Messages|r", "|cffff9933Last|r")

		for _, tm in ipairs(timestamps) do
			local curtm = GetTime() - tm

			if (curtm <= nanoTalk.trackTime) then
				linenum = tip:AddLine(people[tm].name, people[tm].msgs, TimeStr(curtm))
				updatedLines[linenum] = tm
			end
		end
		tip:AddLine(" ")
		linenum = tip:AddLine("Left Click:")
		tip:SetCell(linenum, 2, "Clear list.", tip:GetFont(), "LEFT")
	else
		tip:AddLine("No messages.")
		tip:AddLine(" ")
	end
	linenum = tip:AddLine("Right Click:")
	tip:SetCell(linenum, 2, "Configure.", tip:GetFont(), "LEFT")
	tip:Show()
end


local function UpdateTooltip()
	if (tip:IsShown()) then
		updatedLines = {}
		ShowTooltip()
	end
end


function nanoTalk:Clear()
	timestamps = {}
	people = {}

	if (tip:IsShown()) then
		updatedLines = {}
		ShowTooltip()
	end
	self:OnUpdate()
end


--------------------
-- Event Handlers --
--------------------
nanoTalk:SetScript("OnEvent",
		   function(self, event, ...)
			   if self[event] then return self[event](self, event, ...) end
		   end
	   )


nanoTalk:RegisterEvent("ADDON_LOADED")


function nanoTalk:ADDON_LOADED(event, addon)
	if (addon ~= "nanoTalk") then
		return
	end

	if ((not nanoTalkConfig["Version"]) or (nanoTalkConfig["Version"] < nanoTalkVersion)) then
		tempConfig = nanoTalkDefaults

		for k, v in pairs(nanoTalkConfig) do
			if (nanoTalkDefaults[k]) then
				tempConfig[k] = v
			end
		end
		tempConfig["Version"] = nanoTalkVersion
		nanoTalkConfig = tempConfig
	end

	self.config = nanoTalkConfig
	self:ConfigUpdate()
	self:Show()
	self:RegisterEvent("CHAT_MSG_WHISPER")
	self:UnregisterEvent("ADDON_LOADED")
	self.ADDON_LOADED = nil
end


function nanoTalk:CHAT_MSG_WHISPER(event, msg, player)
	local stamp = GetTime()
	local last

	for tm, plr in pairs(people) do
		if (plr == player) then
			last = tm
			break
		end
	end

	local msgcnt = 0

	if (last ~= nil) then
		msgcnt = people[last].msgs
		people[last] = nil

		for idx, tm in ipairs(timestamps) do
			if (tm == last) then
				table.remove(timestamps, idx)
				break
			end
		end
	end

	people[stamp] = {
		name = player,
		msgs = msgcnt + 1
	}
	table.insert(timestamps, stamp)

	if (#timestamps > 1) then table.sort(timestamps) end

	if ((self.notifyAFK == true) and UnitIsAFK("player")) then
		SendChatMessage("I have been AFK for "..TimeStr(GetTime() - AFKtime)..".", "WHISPER", nil, player)
	elseif ((last == nil) or ((stamp - last) > self.talkTime)) then
		local count = GetCount(self.talkTime) - 1

		if (count < 1) then UpdateTooltip() return end

		if (self.chatNotify == true) then
			local phrase = "Someone else is"

			if (count > 1) then
				phrase = "I have "..count.." other people"
			end

			SendChatMessage("Hello, "..player.."! "..phrase.." talking to me, so I might not answer right away.", "WHISPER", nil, player)
			ChatFrame1:AddMessage("|cffff9933nanoTalk:|r Informed "..player.." of conversation.")
		end
	end
	UpdateTooltip()
end


function nanoTalk:OnUpdate()
	if (#timestamps > self.trackNum) then
		local ppos = table.remove(timestamps)
		people[ppos] = nil
	end

	if (AFKtime == 0) then
		if (UnitIsAFK("player")) then
			AFKtime = GetTime()
		end
	elseif (not UnitIsAFK("player")) then
		ChatFrame1:AddMessage("|cffff9933nanoTalk:|r You were AFK for "..TimeStr(GetTime() - AFKtime)..".")
		AFKtime = 0
	end

	local total = GetCount(self.trackTime)
	local recent = GetCount(self.talkTime)

	if (recent > 0) then
		self.obj.text = ""..total.."t "..recent.."r"
		self.obj.icon = "Interface\\GossipFrame\\PetitionGossipIcon"
	else
		self.obj.text = ""..total.."t "
		self.obj.icon = "Interface\\GossipFrame\\GossipGossipIcon"
	end

	if (tip:IsShown() and #updatedLines) then
		for linenum,tm in pairs(updatedLines) do
			tip:SetCell(linenum, 3, TimeStr(GetTime() - tm), tip:GetFont(), "CENTER")
		end
	end
end


function nanoTalk.obj:OnClick(button)
	if (button == "LeftButton") then
		nanoTalk:Clear()
	else
		InterfaceOptionsFrame_OpenToCategory(nanoTalk.configframe)
	end
end


function nanoTalk:ConfigUpdate()
	self.talkTime = self.config.TalkTime
	self.trackTime = self.config.TrackTime
	self.trackNum = self.config.TrackNum
	self.chatNotify = self.config.ChatNotify
	self.notifyAFK = self.config.NotifyAFK
end


--[[
function nanoTalk:ConfigDefaults()
	self.config = nanoTalkDefaults
	self:ConfigUpdate()
end
]]


function nanoTalk.obj.OnLeave()
	tip:Hide()
	updatedLines = {}
end


function nanoTalk.obj.OnEnter(self)
	tip:SmartAnchorTo(self)
	ShowTooltip()
end


--------------------
-- Initialization --
--------------------
nanoTalk:SetScript("OnUpdate", nanoTalk.OnUpdate)
